/*
 * // Bristleback plugin - Copyright (c) 2010 bristleback.googlecode.com
 * // ---------------------------------------------------------------------------
 * // This program is free software; you can redistribute it and/or modify it
 * // under the terms of the GNU Lesser General Public License as published by the
 * // Free Software Foundation; either version 3 of the License, or (at your
 * // option) any later version.
 * // This library is distributed in the hope that it will be useful,
 * // but without any warranty; without even the implied warranty of merchantability
 * // or fitness for a particular purpose.
 * // You should have received a copy of the GNU Lesser General Public License along
 * // with this program; if not, see <http://www.gnu.org/licenses/lgpl.html>.
 * // ---------------------------------------------------------------------------
 */

package pl.bristleback.server.bristle.integration.spring;

import org.apache.log4j.Logger;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

/**
 * Utility class for resolving Spring dependencies information.
 * <p/>
 * <p/>
 * Created on: 2010-11-18 22:45:23 <br/>
 *
 * @author Wojciech Niemiec
 * @see pl.bristleback.server.bristle.integration.spring.SpringIntegrationUtil
 */
public final class SpringBeanDependenciesResolver {
  private static Logger log = Logger.getLogger(SpringBeanDependenciesResolver.class.getName());

  private SpringBeanDependenciesResolver() {
    throw new UnsupportedOperationException();
  }

  /**
   * Searches for annotated setter methods in given bean class.
   * It is not required to have standard setXXX() method,
   * any method marked with {@link pl.bristleback.server.bristle.integration.spring.BristleSpringBean}
   * will be used as Spring Bean setters.
   *
   * @param beanClass class containing annotated dependencies.
   * @return list of dependency wrappers.
   */
  public static List<SpringBeanDependency> getSpringBeanDependencies(Class beanClass) {
    List<SpringBeanDependency> dependencies = new ArrayList<SpringBeanDependency>();
    for (Method method : getBeansSetters(beanClass)) {
      SpringBeanDependency dependency = createDependency(method);
      dependencies.add(dependency);
    }
    return dependencies;
  }

  private static SpringBeanDependency createDependency(Method setter) {
    BristleSpringBean springBean = setter.getAnnotation(BristleSpringBean.class);
    String ref = springBean.ref();
    Class<?> beanType = setter.getParameterTypes()[0];
    return new SpringBeanDependency(beanType, ref, setter);
  }

  private static List<Method> getBeansSetters(Class beanClass) {
    List<Method> annotatedMethods = new ArrayList<Method>();
    for (Method method : beanClass.getMethods()) {
      if (method.isAnnotationPresent(BristleSpringBean.class)) {
        annotatedMethods.add(method);
      }
    }
    return annotatedMethods;
  }
}
