/*
 * // Bristleback plugin - Copyright (c) 2010 bristleback.googlecode.com
 * // ---------------------------------------------------------------------------
 * // This program is free software; you can redistribute it and/or modify it
 * // under the terms of the GNU Lesser General Public License as published by the
 * // Free Software Foundation; either version 3 of the License, or (at your
 * // option) any later version.
 * // This library is distributed in the hope that it will be useful,
 * // but without any warranty; without even the implied warranty of merchantability
 * // or fitness for a particular purpose.
 * // You should have received a copy of the GNU Lesser General Public License along
 * // with this program; if not, see <http://www.gnu.org/licenses/lgpl.html>.
 * // ---------------------------------------------------------------------------
 */

package pl.bristleback.server.bristle.integration.spring;

import org.apache.log4j.Logger;

import java.lang.reflect.Method;

/**
 * Class holding information about action class dependency.
 * <p/>
 * Created on: 2010-11-18 22:43:40 <br/>
 *
 * @author Wojciech Niemiec
 */
public final class SpringBeanDependency {
  private static Logger log = Logger.getLogger(SpringBeanDependency.class.getName());

  private Class<?> dependencyType;
  private String refName;
  private Method setter;

  public SpringBeanDependency(Class<?> dependencyType, String refName, Method setter) {
    this.dependencyType = dependencyType;
    this.refName = refName;
    this.setter = setter;
  }

  /**
   * Type of dependency. Type is checked while retrieving dependency.
   *
   * @return type of dependency.
   */
  public Class<?> getDependencyType() {
    return dependencyType;
  }

  /**
   * Dependency reference.
   *
   * @return dependency reference.
   */
  public String getRefName() {
    return refName;
  }

  /**
   * Method that sets dependency into bean.
   *
   * @return setter.
   */
  public Method getSetter() {
    return setter;
  }
}
